@@ -17,22 +17,22 @@ class TestTypeAnnotations(TestCase):
         from typing import overload
 
         @overload
-        def f(s):  # type: (None) -> None
+        def f(s: None) -> None:
             pass
 
         @overload
-        def f(s):  # type: (int) -> int
+        def f(s: int) -> int:
             pass
 
         def f(s):
             return s
 
         @typing.overload
-        def g(s):  # type: (None) -> None
+        def g(s: None) -> None:
             pass
 
         @typing.overload
-        def g(s):  # type: (int) -> int
+        def g(s: int) -> int:
             pass
 
         def g(s):
@@ -46,40 +46,39 @@ class TestTypeAnnotations(TestCase):
         from typing_extensions import overload
 
         @overload
-        def f(s):  # type: (None) -> None
+        def f(s: None) -> None:
             pass
 
         @overload
-        def f(s):  # type: (int) -> int
+        def f(s: int) -> int:
             pass
 
         def f(s):
             return s
 
         @typing_extensions.overload
-        def g(s):  # type: (None) -> None
+        def g(s: None) -> None:
             pass
 
         @typing_extensions.overload
-        def g(s):  # type: (int) -> int
+        def g(s: int) -> int:
             pass
 
         def g(s):
             return s
         """)
 
-    @skipIf(version_info < (3, 5), 'new in Python 3.5')
     def test_typingOverloadAsync(self):
         """Allow intentional redefinitions via @typing.overload (async)"""
         self.flakes("""
         from typing import overload
 
         @overload
-        async def f(s):  # type: (None) -> None
+        async def f(s: None) -> None:
             pass
 
         @overload
-        async def f(s):  # type: (int) -> int
+        async def f(s: int) -> int:
             pass
 
         async def f(s):
@@ -93,12 +92,12 @@ class TestTypeAnnotations(TestCase):
 
             @dec
             @overload
-            def f(x):  # type: (int) -> int
+            def f(x: int) -> int:
                 pass
 
             @dec
             @overload
-            def f(x):  # type: (str) -> str
+            def f(x: str) -> str:
                 pass
 
             @dec
@@ -111,16 +110,33 @@ class TestTypeAnnotations(TestCase):
 
         class C:
             @overload
-            def f(self, x):  # type: (int) -> int
+            def f(self, x: int) -> int:
                 pass
 
             @overload
-            def f(self, x):  # type: (str) -> str
+            def f(self, x: str) -> str:
                 pass
 
             def f(self, x): return x
         """)
 
+    def test_aliased_import(self):
+        """Detect when typing is imported as another name"""
+        self.flakes("""
+        import typing as t
+
+        @t.overload
+        def f(s: None) -> None:
+            pass
+
+        @t.overload
+        def f(s: int) -> int:
+            pass
+
+        def f(s):
+            return s
+        """)
+
     def test_not_a_typing_overload(self):
         """regression test for @typing.overload detection bug in 2.1.0"""
         self.flakes("""
@@ -135,7 +151,6 @@ class TestTypeAnnotations(TestCase):
                 pass
         """, m.RedefinedWhileUnused)
 
-    @skipIf(version_info < (3, 6), 'new in Python 3.6')
     def test_variable_annotations(self):
         self.flakes('''
         name: str
@@ -159,7 +174,7 @@ class TestTypeAnnotations(TestCase):
         def f():
             name: str
             age: int
-        ''')
+        ''', m.UnusedAnnotation, m.UnusedAnnotation)
         self.flakes('''
         def f():
             name: str = 'Bob'
@@ -175,7 +190,7 @@ class TestTypeAnnotations(TestCase):
         from typing import Any
         def f():
             a: Any
-        ''')
+        ''', m.UnusedAnnotation)
         self.flakes('''
         foo: not_a_real_type
         ''', m.UndefinedName)
@@ -283,7 +298,50 @@ class TestTypeAnnotations(TestCase):
         a: 'a: "A"'
         ''', m.ForwardAnnotationSyntaxError)
 
-    @skipIf(version_info < (3, 6), 'new in Python 3.6')
+    def test_variable_annotation_references_self_name_undefined(self):
+        self.flakes("""
+        x: int = x
+        """, m.UndefinedName)
+
+    def test_TypeAlias_annotations(self):
+        self.flakes("""
+        from typing_extensions import TypeAlias
+        from foo import Bar
+
+        bar: TypeAlias = Bar
+        """)
+        self.flakes("""
+        from typing_extensions import TypeAlias
+        from foo import Bar
+
+        bar: TypeAlias = 'Bar'
+        """)
+        self.flakes("""
+        from typing_extensions import TypeAlias
+        from foo import Bar
+
+        class A:
+            bar: TypeAlias = Bar
+        """)
+        self.flakes("""
+        from typing_extensions import TypeAlias
+        from foo import Bar
+
+        class A:
+            bar: TypeAlias = 'Bar'
+        """)
+        self.flakes("""
+        from typing_extensions import TypeAlias
+
+        bar: TypeAlias
+        """)
+        self.flakes("""
+        from typing_extensions import TypeAlias
+        from foo import Bar
+
+        bar: TypeAlias
+        """, m.UnusedImport)
+
     def test_annotating_an_import(self):
         self.flakes('''
             from a import b, c
@@ -291,7 +349,6 @@ class TestTypeAnnotations(TestCase):
             print(b)
         ''')
 
-    @skipIf(version_info < (3, 6), 'new in Python 3.6')
     def test_unused_annotation(self):
         # Unused annotations are fine in module and class scope
         self.flakes('''
@@ -299,11 +356,10 @@ class TestTypeAnnotations(TestCase):
         class Cls:
             y: int
         ''')
-        # TODO: this should print a UnusedVariable message
         self.flakes('''
         def f():
             x: int
-        ''')
+        ''', m.UnusedAnnotation)
         # This should only print one UnusedVariable message
         self.flakes('''
         def f():
@@ -311,14 +367,25 @@ class TestTypeAnnotations(TestCase):
             x = 3
         ''', m.UnusedVariable)
 
-    @skipIf(version_info < (3, 5), 'new in Python 3.5')
+    def test_unused_annotation_in_outer_scope_reassigned_in_local_scope(self):
+        self.flakes('''
+        x: int
+        x.__dict__
+        def f(): x = 1
+        ''', m.UndefinedName, m.UnusedVariable)
+
+    def test_unassigned_annotation_is_undefined(self):
+        self.flakes('''
+        name: str
+        print(name)
+        ''', m.UndefinedName)
+
     def test_annotated_async_def(self):
         self.flakes('''
         class c: pass
         async def func(c: c) -> None: pass
         ''')
 
-    @skipIf(version_info < (3, 7), 'new in Python 3.7')
     def test_postponed_annotations(self):
         self.flakes('''
         from __future__ import annotations
@@ -343,7 +410,6 @@ class TestTypeAnnotations(TestCase):
         def g(t: 'T'): pass
         ''')
 
-    @skipIf(version_info < (3, 6), 'new in Python 3.6')
     def test_type_annotation_clobbers_all(self):
         self.flakes('''\
         from typing import TYPE_CHECKING, List
@@ -356,118 +422,6 @@ class TestTypeAnnotations(TestCase):
             __all__: List[str]
         ''')
 
-    def test_typeCommentsMarkImportsAsUsed(self):
-        self.flakes("""
-        from mod import A, B, C, D, E, F, G
-
-
-        def f(
-            a,  # type: A
-        ):
-            # type: (...) -> B
-            for b in a:  # type: C
-                with b as c:  # type: D
-                    d = c.x  # type: E
-                    return d
-
-
-        def g(x):  # type: (F) -> G
-            return x.y
-        """)
-
-    def test_typeCommentsFullSignature(self):
-        self.flakes("""
-        from mod import A, B, C, D
-        def f(a, b):
-            # type: (A, B[C]) -> D
-            return a + b
-        """)
-
-    def test_typeCommentsStarArgs(self):
-        self.flakes("""
-        from mod import A, B, C, D
-        def f(a, *b, **c):
-            # type: (A, *B, **C) -> D
-            return a + b
-        """)
-
-    def test_typeCommentsFullSignatureWithDocstring(self):
-        self.flakes('''
-        from mod import A, B, C, D
-        def f(a, b):
-            # type: (A, B[C]) -> D
-            """do the thing!"""
-            return a + b
-        ''')
-
-    def test_typeCommentsAdditionalComment(self):
-        self.flakes("""
-        from mod import F
-
-        x = 1 # type: F  # noqa
-        """)
-
-    def test_typeCommentsNoWhitespaceAnnotation(self):
-        self.flakes("""
-        from mod import F
-
-        x = 1  #type:F
-        """)
-
-    def test_typeCommentsInvalidDoesNotMarkAsUsed(self):
-        self.flakes("""
-        from mod import F
-
-        # type: F
-        """, m.UnusedImport)
-
-    def test_typeCommentsSyntaxError(self):
-        self.flakes("""
-        def f(x):  # type: (F[) -> None
-            pass
-        """, m.CommentAnnotationSyntaxError)
-
-    def test_typeCommentsSyntaxErrorCorrectLine(self):
-        checker = self.flakes("""\
-        x = 1
-        # type: definitely not a PEP 484 comment
-        """, m.CommentAnnotationSyntaxError)
-        self.assertEqual(checker.messages[0].lineno, 2)
-
-    def test_typeCommentsAssignedToPreviousNode(self):
-        # This test demonstrates an issue in the implementation which
-        # associates the type comment with a node above it, however the type
-        # comment isn't valid according to mypy.  If an improved approach
-        # which can detect these "invalid" type comments is implemented, this
-        # test should be removed / improved to assert that new check.
-        self.flakes("""
-        from mod import F
-        x = 1
-        # type: F
-        """)
-
-    def test_typeIgnore(self):
-        self.flakes("""
-        a = 0  # type: ignore
-        b = 0  # type: ignore[excuse]
-        c = 0  # type: ignore=excuse
-        d = 0  # type: ignore [excuse]
-        e = 0  # type: ignore whatever
-        """)
-
-    def test_typeIgnoreBogus(self):
-        self.flakes("""
-        x = 1  # type: ignored
-        """, m.UndefinedName)
-
-    def test_typeIgnoreBogusUnicode(self):
-        error = (m.CommentAnnotationSyntaxError if version_info < (3,)
-                 else m.UndefinedName)
-        self.flakes("""
-        x = 2  # type: ignore\xc3
-        """, error)
-
-    @skipIf(version_info < (3,), 'new in Python 3')
     def test_return_annotation_is_class_scope_variable(self):
         self.flakes("""
         from typing import TypeVar
@@ -478,7 +432,6 @@ class TestTypeAnnotations(TestCase):
                 return x
         """)
 
-    @skipIf(version_info < (3,), 'new in Python 3')
     def test_return_annotation_is_function_body_variable(self):
         self.flakes("""
         class Test:
@@ -487,7 +440,6 @@ class TestTypeAnnotations(TestCase):
                 return Y
         """, m.UndefinedName)
 
-    @skipIf(version_info < (3, 8), 'new in Python 3.8')
     def test_positional_only_argument_annotations(self):
         self.flakes("""
         from x import C
@@ -495,7 +447,6 @@ class TestTypeAnnotations(TestCase):
         def f(c: C, /): ...
         """)
 
-    @skipIf(version_info < (3,), 'new in Python 3')
     def test_partially_quoted_type_annotation(self):
         self.flakes("""
         from queue import Queue
@@ -531,7 +482,7 @@ class TestTypeAnnotations(TestCase):
     def test_type_cast_literal_str_to_str(self):
         # Checks that our handling of quoted type annotations in the first
         # argument to `cast` doesn't cause issues when (only) the _second_
-        # argument is a literal str which looks a bit like a type annoation.
+        # argument is a literal str which looks a bit like a type annotation.
         self.flakes("""
         from typing import cast
 
@@ -560,7 +511,6 @@ class TestTypeAnnotations(TestCase):
         S = TypeVar('S', int, bound='List[int]')
         """)
 
-    @skipIf(version_info < (3,), 'new in Python 3')
     def test_literal_type_typing(self):
         self.flakes("""
         from typing import Literal
@@ -569,7 +519,6 @@ class TestTypeAnnotations(TestCase):
             return None
         """)
 
-    @skipIf(version_info < (3,), 'new in Python 3')
     def test_literal_type_typing_extensions(self):
         self.flakes("""
         from typing_extensions import Literal
@@ -578,7 +527,6 @@ class TestTypeAnnotations(TestCase):
             return None
         """)
 
-    @skipIf(version_info < (3,), 'new in Python 3')
     def test_annotated_type_typing_missing_forward_type(self):
         self.flakes("""
         from typing import Annotated
@@ -587,7 +535,6 @@ class TestTypeAnnotations(TestCase):
             return None
         """, m.UndefinedName)
 
-    @skipIf(version_info < (3,), 'new in Python 3')
     def test_annotated_type_typing_missing_forward_type_multiple_args(self):
         self.flakes("""
         from typing import Annotated
@@ -596,7 +543,6 @@ class TestTypeAnnotations(TestCase):
             return None
         """, m.UndefinedName)
 
-    @skipIf(version_info < (3,), 'new in Python 3')
     def test_annotated_type_typing_with_string_args(self):
         self.flakes("""
         from typing import Annotated
@@ -605,7 +551,6 @@ class TestTypeAnnotations(TestCase):
             return None
         """)
 
-    @skipIf(version_info < (3,), 'new in Python 3')
     def test_annotated_type_typing_with_string_args_in_union(self):
         self.flakes("""
         from typing import Annotated, Union
@@ -614,7 +559,6 @@ class TestTypeAnnotations(TestCase):
             return None
         """, m.UndefinedName)
 
-    @skipIf(version_info < (3,), 'new in Python 3')
     def test_literal_type_some_other_module(self):
         """err on the side of false-negatives for types named Literal"""
         self.flakes("""
@@ -627,7 +571,6 @@ class TestTypeAnnotations(TestCase):
             return None
         """)
 
-    @skipIf(version_info < (3,), 'new in Python 3')
     def test_literal_union_type_typing(self):
         self.flakes("""
         from typing import Literal
@@ -636,7 +579,6 @@ class TestTypeAnnotations(TestCase):
             return None
         """)
 
-    @skipIf(version_info < (3,), 'new in Python 3')
     def test_deferred_twice_annotation(self):
         self.flakes("""
             from queue import Queue
@@ -647,7 +589,6 @@ class TestTypeAnnotations(TestCase):
                 return None
         """)
 
-    @skipIf(version_info < (3, 7), 'new in Python 3.7')
     def test_partial_string_annotations_with_future_annotations(self):
         self.flakes("""
             from __future__ import annotations
@@ -660,6 +601,20 @@ class TestTypeAnnotations(TestCase):
                 return None
         """)
 
+    def test_forward_annotations_for_classes_in_scope(self):
+        # see #749
+        self.flakes("""
+        from typing import Optional
+
+        def f():
+            class C:
+                a: "D"
+                b: Optional["D"]
+                c: "Optional[D]"
+
+            class D: pass
+        """)
+
     def test_idomiatic_typing_guards(self):
         # typing.TYPE_CHECKING: python3.5.3+
         self.flakes("""
@@ -668,7 +623,7 @@ class TestTypeAnnotations(TestCase):
             if TYPE_CHECKING:
                 from t import T
 
-            def f():  # type: () -> T
+            def f() -> T:
                 pass
         """)
         # False: the old, more-compatible approach
@@ -676,7 +631,7 @@ class TestTypeAnnotations(TestCase):
             if False:
                 from t import T
 
-            def f():  # type: () -> T
+            def f() -> T:
                 pass
         """)
         # some choose to assign a constant and do it that way
@@ -686,7 +641,7 @@ class TestTypeAnnotations(TestCase):
             if MYPY:
                 from t import T
 
-            def f():  # type: () -> T
+            def f() -> T:
                 pass
         """)
 
@@ -700,7 +655,7 @@ class TestTypeAnnotations(TestCase):
                 Protocol = object
 
             class C(Protocol):
-                def f():  # type: () -> int
+                def f() -> int:
                     pass
         """)
 
@@ -734,7 +689,6 @@ class TestTypeAnnotations(TestCase):
             cast(A["E"], [])
         """)
 
-    @skipIf(version_info < (3, 6), 'new in Python 3.6')
     def test_namedtypes_classes(self):
         self.flakes("""
             from typing import TypedDict, NamedTuple
@@ -744,3 +698,102 @@ class TestTypeAnnotations(TestCase):
             class Y(NamedTuple):
                 y: NamedTuple("v", [("vv", int)])
         """)
+
+    @skipIf(version_info < (3, 11), 'new in Python 3.11')
+    def test_variadic_generics(self):
+        self.flakes("""
+            from typing import Generic
+            from typing import TypeVarTuple
+
+            Ts = TypeVarTuple('Ts')
+
+            class Shape(Generic[*Ts]): pass
+
+            def f(*args: *Ts) -> None: ...
+
+            def g(x: Shape[*Ts]) -> Shape[*Ts]: ...
+        """)
+
+    @skipIf(version_info < (3, 12), 'new in Python 3.12')
+    def test_type_statements(self):
+        self.flakes("""
+            type ListOrSet[T] = list[T] | set[T]
+
+            def f(x: ListOrSet[str]) -> None: ...
+
+            type RecursiveType = int | list[RecursiveType]
+
+            type ForwardRef = int | C
+
+            type ForwardRefInBounds[T: C] = T
+
+            class C: pass
+        """)
+
+    @skipIf(version_info < (3, 12), 'new in Python 3.12')
+    def test_type_parameters_functions(self):
+        self.flakes("""
+            def f[T](t: T) -> T: return t
+
+            async def g[T](t: T) -> T: return t
+
+            def with_forward_ref[T: C](t: T) -> T: return t
+
+            def can_access_inside[T](t: T) -> T:
+                print(T)
+                return t
+
+            class C: pass
+        """)
+
+    @skipIf(version_info < (3, 12), 'new in Python 3.12')
+    def test_type_parameters_do_not_escape_function_scopes(self):
+        self.flakes("""
+            from x import g
+
+            @g(T)  # not accessible in decorators
+            def f[T](t: T) -> T: return t
+
+            T  # not accessible afterwards
+        """, m.UndefinedName, m.UndefinedName)
+
+    @skipIf(version_info < (3, 12), 'new in Python 3.12')
+    def test_type_parameters_classes(self):
+        self.flakes("""
+            class C[T](list[T]): pass
+
+            class UsesForward[T: Forward](list[T]): pass
+
+            class Forward: pass
+
+            class WithinBody[T](list[T]):
+                t = T
+        """)
+
+    @skipIf(version_info < (3, 12), 'new in Python 3.12')
+    def test_type_parameters_do_not_escape_class_scopes(self):
+        self.flakes("""
+            from x import g
+
+            @g(T)  # not accessible in decorators
+            class C[T](list[T]): pass
+
+            T  # not accessible afterwards
+        """, m.UndefinedName, m.UndefinedName)
+
+    @skipIf(version_info < (3, 12), 'new in Python 3.12')
+    def test_type_parameters_TypeVarTuple(self):
+        self.flakes("""
+        def f[*T](*args: *T) -> None: ...
+        """)
+
+    @skipIf(version_info < (3, 12), 'new in Python 3.12')
+    def test_type_parameters_ParamSpec(self):
+        self.flakes("""
+        from typing import Callable
+
+        def f[R, **P](f: Callable[P, R]) -> Callable[P, R]:
+            def g(*args: P.args, **kwargs: P.kwargs) -> R:
+                return f(*args, **kwargs)
+            return g
+        """)
